//
//  NisusLogManager.h
//  NisusCore
//
//  Created by Martin on 2019.03.19.
//  Copyright © 2019 Nisus. All rights reserved.
//

#import <Foundation/Foundation.h>
#import <NisusCore/NisusLogDomain.h>


#ifdef ENABLE_NISUS_LOG
	#define NisusLog_AVAILABLE 1
#elif NISUS_DEBUG
	#define NisusLog_AVAILABLE 1
#else
	#define NisusLog_AVAILABLE 0
#endif




#if NisusLog_AVAILABLE

#define NisusLogDomain_Sandbox (@"sandbox")

#define NisusLog_USE_FLUSHING 1
#if NisusLog_USE_FLUSHING
typedef NS_ENUM( NSUInteger, NisusLogFlushMode ) {
	NisusLogFlushMode_Manual,		// flush only when manually initiated, or when the app quits
	NisusLogFlushMode_Periodic,		// flush at regular intervals, eg: every 5 seconds
	NisusLogFlushMode_Aggressive,	// flush every time through the run loop as needed
	NisusLogFlushMode_Immediate,		// flush synchronously, for every log call. This is expensive
};
#endif


@interface NisusLogManager : NSObject

+ (instancetype) instance ;

+ (void) restoreLoggingAtLaunch ;
+ (BOOL) ensureLoggingStarted ;

+ (NSURL*) standardLogFileURL ;
+ (NSURL*) priorLaunchLogFileURL ;

#if NisusLog_USE_FLUSHING
@property NisusLogFlushMode fileFlushMode ;
- (void) flushFile ;
#endif

- (BOOL) isLogging ;
- (BOOL) startLogging ;
- (void) stopLogging ;

- (BOOL) logLine:(NSString*)line ;
- (BOOL) logData:(NSData*)data toFileWithName:(NSString*)fileName ;

@end

#endif // NisusLog_AVAILABLE

