//
//  NisusLogging.h
//  NisusCore
//
//  Created by Martin Wierschin on 2013.10.04.
//  Copyright (c) 2013 Nisus. All rights reserved.
//

#import <NisusCore/NisusDefines.h>
#import <NisusCore/NisusLogManager.h>
#import <NisusCore/NSDate+NisusCore.h>




// Build-style specific Logging
#define NISUS_SEND_DEV_LOG_TO_FILE (0)

#if NISUS_DEV_BUILD
	extern void NisusDevLogOutput( NSString* string );
	extern void NisusDevLogAddBacktraceMatch( NSString* needle );

	#define NisusDevLog_MACRO(str , args...) {										\
		NSString* nisusFinalLogString = [NSString stringWithFormat:str, ## args];	\
		NisusDevLogOutput( nisusFinalLogString );									\
	}

	#define NisusDevLog(str , args...)		NisusDevLog_MACRO(str , ## args)

	#if NISUS_SEND_DEV_LOG_TO_FILE
		#define NisusDebugLog(str , args...)	NisusLog_LOG(str , ## args)
		#define NisusReleaseLog(str , args...)	NisusLog_LOG(str , ## args)
	#else
		#define NisusDebugLog(str , args...)	NisusDevLog_MACRO(str , ## args)
		#define NisusReleaseLog(str , args...)	NisusDevLog_MACRO(str , ## args)
	#endif

#elif NisusLog_AVAILABLE
	#define NisusDevLog(str , args...)
	#define NisusDebugLog(str , args...)	NisusLog_LOG(str , ## args)
	#define NisusReleaseLog(str , args...)	NisusLog_LOG(str , ## args)
#else
	#define NisusDevLog(str , args...)
	#define NisusDebugLog(str , args...)
	#define NisusReleaseLog(str , args...)	NSLog(str , ## args)
#endif




#if NisusLog_AVAILABLE
	extern void NisusLog_LogLine( NSString* logLine );

	#define NisusLog_LOG( logFormat, logFormatArgs...)											\
	{																							\
		NSString* _nisusLogLine = [NSString stringWithFormat:(logFormat), ## logFormatArgs];	\
		NisusLog_LogLine( _nisusLogLine);														\
	}

	#define NisusLog_LOG_IF( isLogThis, logFormat, logFormatArgs...)	\
		if( isLogThis ) {												\
			NisusLog_LOG( logFormat, logFormatArgs );					\
		}

	#define NisusLog_BACKTRACE							([NisusLog loggableStackBacktrace])
	#define NisusLog_BACKTRACE_IF( isLogBacktrace )		((isLogBacktrace) ? [NisusLog loggableStackBacktrace] : @"")

#else
	#define NisusLog_LogLine( logLine )
	#define NisusLog_LOG( logFormat, logFormatArgs...)
	#define NisusLog_LOG_IF( isLogThis, logFormat, logFormatArgs...)
#endif




// Flood control. Use the defined MACRO like an IF statement, wrapping the logging block in curly braces, eg:
//
//	NisusLog_FLOOD_CONTROLLED {
//		NisusDevLog( @"some logging goes here" );
//	}

#define NisusLog_FLOOD_CONTROLLED											\
	static NisusLogFloodControlData nisusLogFloodData = {0};				\
	if( _NisusLog_ShouldLogWithFloodControlData(&nisusLogFloodData) )

typedef struct {
	NisusTimeMark lastLogTime;
	NisusTimeMark disableUntilTime;
	UInt32 floodCount;
} NisusLogFloodControlData;

extern BOOL _NisusLog_ShouldLogWithFloodControlData( NisusLogFloodControlData* dataPtr );







@interface NisusLog : NSObject

#if NISUS_DEV_BUILD
+ (void) immortalizeDebugString:(NSString*)str ;
+ (NSString*) immortalizedStackBacktrace ;
#endif

#if NISUS_DEBUG
+ (NSString*) loggableStackBacktrace ;
#endif

@end



